@@ -665,7 +665,8 @@ def process_servicebinding(cpa,
                           partyrole,
                           counterpartyidL,
                           counterpartyrole,
-                          pmode)
+                          pmode,
+                          send_or_receive)
         process_ebms_authorization(cpa, channelid, '1', 'Initiator', pmode)
 
         pmodes.append(pmode)
@@ -695,7 +696,19 @@ def process_partyinfo(partyidL,
                       partyrole,
                       counterpartyidL,
                       counterpartyrole,
-                      pmode):
+                      pmode,
+                      send_or_receive):
+    if send_or_receive == 'send':
+        first_sender_party_idL = partyidL
+        first_sender_party_role = partyrole
+        first_receiver_party_idL = counterpartyidL
+        first_receiver_party_role = counterpartyrole
+    else:
+        first_sender_party_idL = counterpartyidL
+        first_sender_party_role = counterpartyrole
+        first_receiver_party_idL = partyidL
+        first_receiver_party_role = partyrole
+
     if 'Initiator' not in pmode:
         pmode['Initiator'] = {}
     if 'Responder' not in pmode:
@@ -704,42 +717,42 @@ def process_partyinfo(partyidL,
         if pmode['MEPBinding'] == uri_push:
             #pmode['Initiator']['Party'] = partyidL
             process_party_identifier_and_type(pmode['Initiator'],
-                                              partyidL)
-            pmode['Initiator']['Role'] = partyrole
+                                              first_sender_party_idL)
+            pmode['Initiator']['Role'] = first_sender_party_role
             #pmode['Responder']['Party'] = counterpartyidL
             process_party_identifier_and_type(pmode['Responder'],
-                                              counterpartyidL)
-            pmode['Responder']['Role'] = counterpartyrole
+                                              first_receiver_party_idL)
+            pmode['Responder']['Role'] = first_receiver_party_role
         else:
             #pmode['Initiator']['Party'] = counterpartyidL
             process_party_identifier_and_type(pmode['Initiator'],
-                                              counterpartyidL)
-            pmode['Initiator']['Role'] = counterpartyrole
+                                              first_receiver_party_idL)
+            pmode['Initiator']['Role'] = first_receiver_party_role
             #pmode['Responder']['Party'] = partyidL
             process_party_identifier_and_type(pmode['Responder'],
-                                              partyidL)
-            pmode['Responder']['Role'] = partyrole
+                                              first_sender_party_idL)
+            pmode['Responder']['Role'] = first_sender_party_role
     elif pmode['MEP'] == uri_twoway:
         if pmode['MEPBinding'] in (uri_pushandpush,
             uri_pushandpull,
             uri_sync):
             #pmode['Initiator']['Party'] = partyidL
             process_party_identifier_and_type(pmode['Initiator'],
-                                              partyidL)
-            pmode['Initiator']['Role'] = partyrole
+                                              first_sender_party_idL)
+            pmode['Initiator']['Role'] = first_sender_party_role
             #pmode['Responder']['Party'] = counterpartyidL
             process_party_identifier_and_type(pmode['Responder'],
-                                              counterpartyidL)
-            pmode['Responder']['Role'] = counterpartyrole
+                                              first_receiver_party_idL)
+            pmode['Responder']['Role'] = first_receiver_party_role
         else:
             #pmode['Initiator']['Party'] = counterpartyidL
             process_party_identifier_and_type(pmode['Initiator'],
-                                              counterpartyidL)
-            pmode['Initiator']['Role'] = counterpartyrole
+                                              first_receiver_party_idL)
+            pmode['Initiator']['Role'] = first_receiver_party_role
             #pmode['Responder']['Party'] = partyidL
             process_party_identifier_and_type(pmode['Responder'],
-                                              partyidL)
-            pmode['Responder']['Role'] = partyrole
+                                              first_sender_party_idL)
+            pmode['Responder']['Role'] = first_sender_party_role
 
 def process_party_identifier_and_type(parent, party_list):
     parent['Party'] = []
@@ -903,8 +916,7 @@ def process_payloadservice(cpa, pmode, leg, channel):
                     'CompressionType' : 'application/gzip'
                 }
             else:
-                logging.error(package_id+' '+str(len(as4compressedparts))+str(swapackage_config))
-
+                logging.info('No payloadservice in package {}'.format(package_id))
 
 
 def process_reception_awareness(cpa, pmode, leg, channel):
