@@ -13,7 +13,7 @@ import logging, os
 
 logging.basicConfig(level=logging.DEBUG)
 
-_NSMAP = { 'cppa' : 'http://docs.oasis-open.org/ebcore/ns/cppa/v3.0',
+_NSMAP = { 'cppa': 'http://docs.oasis-open.org/ebcore/ns/cppa/v3.0',
            'pycppa3' : 'https://pypi.python.org/pypi/cppa3'}
 
 
@@ -21,6 +21,12 @@ class ChannelProfileHandler():
 
     def __init__(self, config=None):
         self._channel_profiles = {}
+        self._channel_profiles_send = {}
+        self._channel_profiles_receive = {}
+        self._channels = {}
+        self._channel_dependencies = {}
+        self._channel_dependencies_send = {}
+        self._channel_dependencies_receive = {}
         if config != None:
             self.load_profile_config(config)
 
@@ -28,26 +34,92 @@ class ChannelProfileHandler():
         channels = parsed_config_document.xpath('//pycppa3:ChannelProfiles/*',
                                                 namespaces=_NSMAP)
         for channel in channels:
+
+            if _pycppa3('direction') in channel.attrib:
+                direction = channel.get(_pycppa3('direction'))
+                del channel.attrib[_pycppa3('direction')]
+            else:
+                direction = None
+
             channel_profile = channel.xpath('child::cppa:ChannelProfile/text()',
                                             namespaces = _NSMAP)[0]
-            self._channel_profiles[channel_profile] = channel
+            if not direction:
+                self._channel_profiles[channel_profile] = channel
+                channel_dependencies = self._channel_dependencies
+            elif direction == 'send':
+                self._channel_profiles_send[channel_profile] = channel
+                channel_dependencies = self._channel_dependencies_send
+            elif direction == 'receive':
+                self._channel_profiles_receive[channel_profile] = channel
+                channel_dependencies = self._channel_dependencies_receive
+
+            if 'id' in channel.attrib:
+                self._channels[channel.get('id')] = channel
+
+            if channel_profile not in channel_dependencies:
+                channel_dependencies[channel_profile] = []
+
+            for dependent_channel_xpath in [
+                'child::cppa:ReceiptHandling/cppa:ReceiptChannelId/text()',
+                'child::cppa:ErrorHandling/cppa:SenderErrorsReportChannelId/text()',
+                'child::cppa:ErrorHandling/cppa:ReceiverErrorsReportChannelId/text()',
+                'child::cppa:PullHandling/cppa:PullChannelId/text()',
+                'child::cppa:AlternateChannel/text()',
+            ]:
+                dependent_channel_list = channel.xpath(
+                    dependent_channel_xpath,
+                    namespaces = _NSMAP)
+                for dependent_channel in dependent_channel_list:
+                    if dependent_channel not in channel_dependencies[channel_profile]:
+                        channel_dependencies[channel_profile].append(dependent_channel)
 
     def apply_profile_configs(self, indoc):
         outdoc = etree.Element(indoc.tag, nsmap = {
             'cppa':'http://docs.oasis-open.org/ebcore/ns/cppa/v3.0' })
-        for child in indoc:
+        included_dependent_channels = []
+        logging.debug('@@@@@@@@@@@ processing {}'.format(indoc.tag))
+        for child in deepcopy(indoc):
+            applicable_channel_profiles = self._channel_profiles
+            applicable_channel_dependencies = self._channel_dependencies
+            if 'id' in child.attrib:
+                element_id = child.get('id')
+                if indoc.xpath(
+                    'descendant::cppa:ActionBinding[@sendOrReceive="send"]/cppa:ChannelId="{}"'.format(element_id),
+                    namespaces = _NSMAP
+                ):
+                    applicable_channel_profiles = self._channel_profiles_send
+                    applicable_channel_dependencies = self._channel_dependencies_send
+                    logging.info('@@ Channel {} is a SEND channel'.format(element_id))
+                elif indoc.xpath(
+                    'descendant::cppa:ActionBinding[@sendOrReceive="receive"]/cppa:ChannelId="{}"'.format(element_id),
+                    namespaces = _NSMAP
+                ):
+                    applicable_channel_profiles = self._channel_profiles_receive
+                    applicable_channel_dependencies = self._channel_dependencies_receive
+                    logging.info('@@ Channel {} is a RECEIVE channel'.format(element_id))
+                else:
+                    logging.info('@@ Channel {} is neither a SEND nor a RECEIVE channel'.format(element_id))
+
             if len(child.xpath('cppa:ChannelProfile',
-                               namespaces = _NSMAP)):
+                               namespaces = _NSMAP)) > 0:
                 channel_profile = child.xpath('child::cppa:ChannelProfile/text()',
                                                 namespaces = _NSMAP)[0]
-                if channel_profile in self._channel_profiles:
-                    default = self._channel_profiles[channel_profile]
+                if channel_profile in applicable_channel_profiles:
+                    default = applicable_channel_profiles[channel_profile]
                     if child.tag != default.tag:
                         raise Exception('{} != {}'.format(child.tag, default.tag))
                     logging.info('Document used a channel profile {}'.format(channel_profile))
                     child = apply_default_to_complex_subelement(child,
-                                                                  default)
+                                                                default)
+                for dependent_channel in applicable_channel_dependencies[channel_profile]:
+                    if dependent_channel not in included_dependent_channels:
+                        logging.info('Adding dependent channel {}'.format(dependent_channel))
+                        outdoc.append(deepcopy(self._channels[dependent_channel]))
+                        included_dependent_channels.append(dependent_channel)
             outdoc.append(child)
+        import lxml.etree
+        logging.debug('RESULT @@@: {}'.format(lxml.etree.tostring(outdoc,
+                                                                  pretty_print=True)))
         return outdoc
 
 def cppa(el):
@@ -68,6 +140,7 @@ cppa3_content_model = {
                             cppa('ErrorHandling'),
                             cppa('ReceiptHandling'),
                             cppa('PullHandling'),
+                            cppa('Compression'),
                             cppa('Bundling'),
                             cppa('Splitting'),
                             cppa('AlternateChannel')],
@@ -100,8 +173,8 @@ cppa3_content_model = {
                           cppa('SignAttachments'),
                           cppa('SignExternalPayloads')],
 
-    cppa('Encryption') : [ cppa('DataEncryption'),
-                           cppa('KeyEncryption'),
+    cppa('Encryption') : [ cppa('KeyEncryption'),
+                           cppa('DataEncryption'),
                            cppa('EncryptionCertificateRef'),
                            cppa('EncryptionCertificateRefType'),
                            cppa('EncryptionTrustAnchorRef')],
@@ -139,7 +212,7 @@ def apply_default_to_complex_subelement(element, default):
         if len(element_children) == 0:
             logging.info('{} children from default'.format(tag))
             for child in default_children:
-                if default_node.get(pycppa3('ifused')) == 'true':
+                if child.get(pycppa3('ifused')) == 'true':
                     logging.info('Skipped {} due to pycppa3="https://pypi.python.org/pypi/cppa3"'.format(child.tag))
                 else:
                     result.append(child)
@@ -160,7 +233,7 @@ def apply_default_to_complex_subelement(element, default):
                     #apply_attribute_defaults(element_node, default_node)
                     element_content_model_position = cppa3_content_model[tag].index(element_node.tag)
                     default_content_model_position = cppa3_content_model[tag].index(default_node.tag)
-                    logging.debug('@@ {} {} {} {} {} {}'.format(element_node.tag,
+                    logging.debug('{} {} {} {} {} {}'.format(element_node.tag,
                                                                 element_counter,
                                                                 element_content_model_position,
                                                                 default_node.tag,
@@ -174,6 +247,8 @@ def apply_default_to_complex_subelement(element, default):
                                                                            default_node))
                         element_counter += 1
                         default_counter += 1
+                        already_processed.append(element_node.tag)
+
 
                     elif element_content_model_position < default_content_model_position:
                         logging.debug('Y {} [{},{}]'.format(element_node.tag,
@@ -184,6 +259,7 @@ def apply_default_to_complex_subelement(element, default):
                                                                                         default_content_model_position))
                         result.append(element_node)
                         element_counter += 1
+                        already_processed.append(element_node.tag)
 
                     elif element_content_model_position > default_content_model_position:
                         logging.debug('Z {} [{},{}]'.format(element_node.tag,
@@ -223,7 +299,10 @@ def apply_default_to_complex_subelement(element, default):
                 elif default_counter == len(default_children):
                     logging.info('B Remaining {} children from element'.format(tag))
                     for child in element_children[element_counter:]:
-                        result.append(child)
+                        if child.tag in already_processed:
+                            logging.info('Skipped already processed {}'.format(child.tag))
+                        else:
+                            result.append(child)
                     element_counter = len(element_children)+1
 
                 previous_element = element_node.tag
@@ -241,4 +320,8 @@ def apply_attribute_defaults(provided_element, default_element):
             logging.debug('On {} setting att {} to {} {}'.format(provided_element.tag,
                                                               att,
                                                               default_element.get(att),
-                                                              provided_element.tag))
\ No newline at end of file
+                                                              provided_element.tag))
+
+def _pycppa3(el):
+    return '{{{}}}{}'.format(_NSMAP['pycppa3'],el)
+
