@@ -18,7 +18,7 @@ class CPPA3TestCase( unittest.TestCase ):
     def setUp(self):
         logging.getLogger('').handlers = []
         logging.basicConfig(level=logging.DEBUG,
-                            filename="cppa3_test.log")
+                            filename="unify_test.log")
         thisdir = dirname(abspath(getsourcefile(lambda:0)))
         self.testdatadir = join(thisdir,'data')
         self.testconfigdir = join(thisdir,'config')
@@ -59,24 +59,46 @@ class CPPA3TestCase( unittest.TestCase ):
         cppb_file = os.path.join(self.testdatadir,'cpp_b_'+id+'.xml')
         acpp =  (lxml.etree.parse(cppa_file, self.parser)).getroot()
         bcpp =  (lxml.etree.parse(cppb_file, self.parser)).getroot()
-        if self.schema != None:
+        if self.schema is not None:
+            # Change the serial number as many CAs generate values that are
+            # too long
+            for tmpcpp in [deepcopy(acpp),
+                           deepcopy(bcpp)]:
+                for serialnum in tmpcpp.xpath(
+                    'descendant::ds:X509SerialNumber',
+                    namespaces = { 'ds': 'http://www.w3.org/2000/09/xmldsig#'}
+                ):
+                    serialnum.text = '1234'
+                if not self.schema.validate(tmpcpp):
+                    err = str(self.schema.error_log.last_error)
+                    raise cppa3.unify.UnificationException('Invalid input: {}'.format(err))
+
             if self.schema.validate(acpp) and self.schema.validate(bcpp):
                 logging.info('CPP A and B are valid for {}'.format(id))
-            else:
-                err = str(self.schema.error_log.last_error)
-                raise cppa3.unify.UnificationException('Invalid input: {}'.format(err))
+
         cpa = unify(acpp, bcpp, agreementid=agreementid,
                     requested_activation_date=requested_activation_date,
                     requested_expiration_date=requested_expiration_date,
                     acpp_url=acpp_url,
                     bcpp_url=bcpp_url,
                     default_handler = self.channel_profile_handler,
-                    handle_defaults=handle_defaults
-                    )
+                    handle_defaults=handle_defaults,
+                    delegation_handler=is_connected_to
+        )
         if self.schema is not None:
-            if not self.schema.validate(cpa):
+            # Change the serial number as many CAs generate values that are
+            # too long
+            tmpcpa = deepcopy(cpa)
+
+            for serialnum in tmpcpa.xpath(
+                    'descendant::ds:X509SerialNumber',
+                    namespaces = { 'ds': 'http://www.w3.org/2000/09/xmldsig#'}
+            ):
+                serialnum.text = '1234'
+            if not self.schema.validate(tmpcpa):
                 err = str(self.schema.error_log.last_error)
                 logging.error('CPA {} is invalid: {}'.format(id, err))
+                logging.error(lxml.etree.tostring(cpa, pretty_print=True))
                 raise cppa3.unify.UnificationException('CPA {} is invalid: {}'.format(id, err))
             else:
                 logging.info('Generated CPA valid for {}'.format(id))
@@ -85,7 +107,7 @@ class CPPA3TestCase( unittest.TestCase ):
 
         cpa_file_to_test_against = os.path.join(self.testdatadir,'cppa_ab_'+id+'.xml')
         if not self.replacemode and os.path.isfile(cpa_file_to_test_against):
-            base_cpa =  lxml.etree.parse(cpa_file_to_test_against)
+            base_cpa = lxml.etree.parse(cpa_file_to_test_against)
             base_cpa_clean = reset_timestamped_values_for_compare(base_cpa)
             base_cpa_clean_string = lxml.etree.tostring(base_cpa_clean,
                                                         pretty_print = True)
@@ -106,10 +128,6 @@ class CPPA3TestCase( unittest.TestCase ):
 
         logging.info('------------------------------------------')
 
-
-
-
-
     """
     General tests using NamedProtocolBinding
     """
@@ -211,7 +229,6 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_0028(self):
         self.do_unification('0028')
 
-
     # Tests to show that unification is not commutative, illustrating depth-first
     # backtracking search
     def test_0030(self):
@@ -227,13 +244,13 @@ class CPPA3TestCase( unittest.TestCase ):
     # test with activation date set
     def test_0041(self):
         import datetime
-        requested_date = now = datetime.datetime(2050,10,5,0,0,0)
+        requested_date = datetime.datetime(2050,10,5,0,0,0)
         self.do_unification('0041',  requested_activation_date=requested_date)
 
     # test with expiration date set
     def test_0042(self):
         import datetime
-        requested_date = now = datetime.datetime(2050,10,5,0,0,0)
+        requested_date = datetime.datetime(2050,10,5,0,0,0)
         self.do_unification('0042',  requested_expiration_date=requested_date)
 
     # test with CPP URLs
@@ -251,7 +268,6 @@ class CPPA3TestCase( unittest.TestCase ):
         self.assertRaises( cppa3.unify.UnificationException, self.do_unification, '0211' )
 
     # Named Channel variant of 1007
-    @unittest.skip
     def test_0060(self):
         self.do_unification('0060')
 
@@ -297,12 +313,10 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_0106(self):
         self.do_unification('0106')
 
-
     # Mismatch in number of parts
     def test_0107(self):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '0107')
 
-
     # Payload signature
     def test_0110(self):
         self.do_unification('0110')
@@ -313,12 +327,10 @@ class CPPA3TestCase( unittest.TestCase ):
 
     # Payload encryption
 
-
     """
     0200 ...  reserved for future use ...
     """
 
-
     """
     0300 Certificates and Trust Anchors for Signing, Encryption and TLS
     """
@@ -511,8 +523,6 @@ class CPPA3TestCase( unittest.TestCase ):
     ebMS3 tests
     """
 
-    # ebMS3 test
-
     def test_1000(self):
         # two delivery channels,  the first one is compatible and chosen
         self.do_unification('1000')
@@ -537,7 +547,6 @@ class CPPA3TestCase( unittest.TestCase ):
         # two delivery channels,  the first one is non compatible so the second must be chosen
         self.do_unification('1006')
 
-    @unittest.skip
     def test_1007(self):
         # example involving wider variety of services
         self.do_unification('1007')
@@ -554,7 +563,6 @@ class CPPA3TestCase( unittest.TestCase ):
         # Variant of 1011 that uses channel features
         self.do_unification('1012')
 
-
     def test_1020(self):
         # Variant of 1011 that uses channel features
         self.do_unification('1020')
@@ -732,7 +740,6 @@ class CPPA3TestCase( unittest.TestCase ):
 
     # AS4 and ReceiptHandling, Synchronous, Non-Repudiation
 
-
     # AS4 ReceptionAwareness channel feature
     # Both sides
     def test_1290(self):
@@ -746,10 +753,25 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_1292(self):
         self.do_unification('1292')
 
-
     # 1300 range
     # Compression and Packaging
 
+    # Both have one CompressionType with compatible values
+    def test_1300(self):
+        self.do_unification('1300')
+
+    # Both have one CompressionType but the values are not compatible
+    def test_1301(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1301')
+
+    # Both have multiple CompressionTypes and one value is compatible
+    def test_1305(self):
+        self.do_unification('1305')
+
+    # Both have multiple CompressionTypes but the values are not compatible
+    def test_1310(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1310')
+
     # 1400 range
     # Advanced features
 
@@ -761,7 +783,6 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_1410(self):
         self.do_unification('1410')
 
-
     # 1500, 1501, 1502, 1503 Multi Hop:  actorOrRole attribute present on none or on both,
     # and if on both with same value
     def test_1500(self):
@@ -776,7 +797,7 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_1503(self):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1503')
 
-    # 1505 Addressing present, inline
+    # 1504 Addressing present, inline
     def test_1504(self):
         self.do_unification('1504')
 
@@ -784,7 +805,7 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_1505(self):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1505')
 
-    # 1505 Addressing in channel present in both but inconsistent Action
+    # 1506 Addressing in channel present in both but inconsistent Action
     def test_1506(self):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1506')
 
@@ -813,9 +834,36 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_1512(self):
         self.do_unification('1512')
 
-    # 1515 combination of sorts
+    # 1515 combination of sorts,  fails because of mismatch in ActionSuffix
     def test_1515(self):
-        self.do_unification('1515')
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1515')
+
+    # wsa:From
+    def test_1520(self):
+        self.do_unification('1520')
+
+    # Bundling,  present on both sides, no Policy,  no Scope
+    def test_1530(self):
+        self.do_unification('1530')
+
+    # Present on one side only,  fails
+    def test_1531(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1531')
+
+    # Present on one side only,  fails
+
+    # Ordering present on both sides, compatible
+    def test_1533(self):
+        # Policy on both sides,  compatible
+        self.do_unification('1533')
+
+    # Ordering present on one side only, incompatible
+    def test_1534(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1534')
+
+    # Scope ?
+
+    # @@@ TO DO:  Split / Join / Compress
 
     # 1600 ENTSOG AS4 tests
 
@@ -823,9 +871,50 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_1600(self):
         self.do_unification('1600')
 
+    # Use of ChannelProfile and defined defaults for sending and receiving
+    # Also, the defaults specify channels for signals.  Those channels are
+    # also added to the CPA.
     def test_1601(self):
         self.do_unification('1601', handle_defaults=True)
 
+    # Variant of 1601 that has default signing and encryption certificates in CPP A
+    def test_1602(self):
+        self.do_unification('1602', handle_defaults=True)
+
+    # Variant of 1601 that has default signing and encryption certificates in CPP B
+    def test_1603(self):
+        self.do_unification('1603', handle_defaults=True)
+
+    # Variant of 1601 that has default signing and encryption certificates in CPP A and B
+    def test_1604(self):
+        self.do_unification('1604', handle_defaults=True)
+
+
+    # 1900 SAML Conformance Clause
+
+    # One IDP on both sides, matching
+    def test_1900(self):
+        self.do_unification('1900')
+
+    # Multiple IDPs on both sides, one matching
+    def test_1901(self):
+        self.do_unification('1901')
+
+    # Multiple IDPs on both sides, none matching
+    def test_1902(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1902')
+
+    # One IDP on both sides, mismatch in SAML version
+    def test_1903(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1903')
+
+    # One IDP on both sides, mismatch in KeyType
+    def test_1904(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1904')
+
+    # One IDP on both sides, required SAML Attribute not provided by sender
+    def test_1905(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '1905')
 
     """
     Web Services Tests
@@ -876,7 +965,6 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_2114(self):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '2114')
 
-
     # Multiple SignatureAlgorithms
     # All considered equally acceptable
     def test_2115(self):
@@ -906,7 +994,6 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_2121(self):
         self.do_unification('2121')
 
-
     # @@@ add some variations
 
     # SignAttachments
@@ -921,7 +1008,6 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_2127(self):
         self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '2127')
 
-
     # Encryption
     # Base case, data encryption and certificate reference
     def test_2140(self):
@@ -986,24 +1072,76 @@ class CPPA3TestCase( unittest.TestCase ):
     EDIINT tests
     """
 
-    # 3000 range
-
-    # AS1
-
-    # AS2
+    # AS2 single payload with signing, no receipt requested
     def test_3000(self):
         self.do_unification('3000')
 
+    # AS2 single payload with signing and a synchronous signed receipt
+    def test_3001(self):
+        self.do_unification('3001')
 
-    # AS3
+    # AS2 single payload with signing and a synchronous unsigned receipt
+    def test_3002(self):
+        self.do_unification('3002')
 
-    # Signing
+    # AS2 single payload with signing and an asynchronous signed receipt
+    def test_3003(self):
+        self.do_unification('3003')
 
-    # Encryption
+    # AS2 single payload with signing and an asynchronous signed AS1 receipt
+    def test_3004(self):
+        self.do_unification('3004')
 
-    # Signing and Encryption
+    # AS2 compressed single payload with signing, no receipt requested
+    def test_3050(self):
+        self.do_unification('3050')
 
-    # Compression
+    # AS1 example
+    def test_3100(self):
+        self.do_unification('3100')
+
+    # AS3 example,  sender puts to receiver
+    def test_3200(self):
+        self.do_unification('3200')
+
+    # AS3 example,  receiver gets from receiver
+    def test_3201(self):
+        self.do_unification('3201')
+
+    # AS2 with multiple uncompressed payloads, RFC 6362
+    def test_3075(self):
+        self.do_unification('3075')
+
+    # AS2 with multiple compressed payloads
+    def test_3076(self):
+        self.do_unification('3076')
+
+    # AS3 example with multiple compressed payloads
+    def test_3210(self):
+        self.do_unification('3210')
+
+    # CTE:  Chunked Transfer Encoding
+    def test_3300(self):
+        self.do_unification('3300')
+
+    # SHA-2: Various SHA2 algorithms
+    def test_3305(self):
+        self.do_unification('3305')
+
+    # Filename Preservation
+    def test_3310(self):
+        self.do_unification('3310')
+
+    # AS2 restart
+    def test_3320(self):
+        self.do_unification('3320')
+
+    # RestartInterval set by Receiver
+    def test_3321(self):
+        self.do_unification('3321')
+
+    # CEM.  Could be done by defining a CEM service and
+    # related actions, as done for ebCore Agreement Update.
 
     """
     ebMS2 tests
@@ -1048,8 +1186,6 @@ class CPPA3TestCase( unittest.TestCase ):
     def test_4031(self):
         self.do_unification('4031')
 
-
-
     # Synchronous Business Response
 
     # Two Way MEP,  asynchronous
@@ -1064,8 +1200,37 @@ class CPPA3TestCase( unittest.TestCase ):
     Extensibility
 
     """
-    #def test_0100(self):
-    #    self.do_unification('0100')
+
+    """
+    Delegation Channels
+
+    """
+
+    # Party Delegates,  CounterParty uses AS2
+    def test_5000(self):
+        self.do_unification('5000')
+
+    # Party has AS2 channel, CounterParty delegates
+    def test_5001(self):
+        self.do_unification('5001')
+
+    # Party and CounterParty delegate to same third party
+    def test_5002(self):
+        self.do_unification('5002')
+
+    # Party and CounterParty delegate to different parties, compatible
+    def test_5003(self):
+        self.do_unification('5003')
+
+    # Party and CounterParty delegate to different parties, incompatible
+    def test_5004(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '5004')
+
+    # Party Delegates,  CounterParty uses AS2, incompatible
+    def test_5005(self):
+        self.assertRaises(cppa3.unify.UnificationException, self.do_unification, '5005')
+
+
 
     """
     Tests for lower level functions
@@ -1336,8 +1501,37 @@ def reset_timestamped_values_for_compare(cpa):
                 el.text = 'NONE'
     return cpa_to_edit
 
+def is_connected_to(
+        frompartyid,
+        frompartytype,
+        fromcppid,
+        service,
+        action,
+        direction,
+        topartyid,
+        topartyidtype,
+        topartycppid):
 
+    logging.info('Called with {} {} {} {} {} {}'.format(frompartyid, frompartytype, fromcppid,
+                                                        topartyid, topartyidtype, topartycppid))
 
-
-
-
+    if frompartyid == topartyid \
+        and frompartytype == topartyidtype:
+        return True
+    if frompartyid == 'C' \
+        and frompartytype == 'urn:oasis:names:tc:ebcore:partyid-type:unregistered' \
+        and topartyid == 'B' \
+        and topartyidtype == 'urn:oasis:names:tc:ebcore:partyid-type:unregistered':
+        return True
+    elif frompartyid == 'A' \
+        and frompartytype == 'urn:oasis:names:tc:ebcore:partyid-type:unregistered' \
+        and topartyid == 'D' \
+        and topartyidtype == 'urn:oasis:names:tc:ebcore:partyid-type:unregistered':
+        return True
+    elif frompartyid == 'C' \
+        and frompartytype == 'urn:oasis:names:tc:ebcore:partyid-type:unregistered' \
+        and topartyid == 'D' \
+        and topartyidtype == 'urn:oasis:names:tc:ebcore:partyid-type:unregistered':
+        return True
+    else:
+        return False
